import subprocess

CHROMECAST_CMD = 'stream2chromecast'
COMMON_PLAYERS = ['xdg-open', 'bomi', 'vlc', 'smplayer', 'totem', 'mpv', 'mplayer']


def execute(args):
    p = subprocess.Popen(args)
    p.communicate()


def chromecast(path, cmd=None):
    cmd = cmd or CHROMECAST_CMD
    args = [cmd]
    if not path.endswith('mp4'):
        args += ['-transcode']
    execute(args + [path])


def native(path, cmd=None):
    if cmd is None:
        cmds = COMMON_PLAYERS
    else:
        cmds = [cmd]
    for cmd in cmds:
        try:
            execute([cmd] + [path])
        except OSError:
            pass
        else:
            return


def exec_player(path, type='native', cmd=None):
    return globals()[type](path, cmd)



"""Amazon Dash.
"""

import argparse
import logging

import sys

from random_episode.main import RandomEpisode

CONFIG_FILE = 'random-episode.yml'


def create_logger(name, level=logging.INFO):
    # create logger
    logger = logging.getLogger(name)
    logger.setLevel(level)

    # create console handler and set level to debug
    ch = logging.StreamHandler()
    ch.setLevel(level)

    # create formatter
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)-7s - %(message)s')

    # add formatter to ch
    ch.setFormatter(formatter)

    # add ch to logger
    logger.addHandler(ch)


def set_default_subparser(self, name, args=None):
    """default subparser selection. Call after setup, just before parse_args()
    name: is the name of the subparser to call by default
    args: if set is the argument list handed to parse_args()
    , tested with 2.7, 3.2, 3.3, 3.4
    it works with 2.6 assuming argparse is installed
    """
    subparser_found = False
    for arg in sys.argv[1:]:
        if arg in ['-h', '--help']:  # global help if no subparser
            break
    else:
        for x in self._subparsers._actions:
            if not isinstance(x, argparse._SubParsersAction):
                continue
            for sp_name in x._name_parser_map.keys():
                if sp_name in sys.argv[1:]:
                    subparser_found = True
        if not subparser_found:
            # insert default in first position, this implies no
            # global options without a sub_parsers specified
            if args is None:
                sys.argv.insert(1, name)
            else:
                args.insert(0, name)

argparse.ArgumentParser.set_default_subparser = set_default_subparser


def execute_from_command_line(argv=None):
    """
    A simple method that runs a ManagementUtility.
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('--config', default=CONFIG_FILE)

    parser.add_argument('--warning', help='set logging to warning', action='store_const', dest='loglevel',
                        const=logging.WARNING, default=logging.INFO)
    parser.add_argument('--quiet', help='set logging to ERROR', action='store_const', dest='loglevel',
                        const=logging.ERROR, default=logging.INFO)
    parser.add_argument('--debug', help='set logging to DEBUG',
                        action='store_const', dest='loglevel',
                        const=logging.DEBUG, default=logging.INFO)
    parser.add_argument('--verbose', help='set logging to COMM',
                        action='store_const', dest='loglevel',
                        const=5, default=logging.INFO)
    parser.sub = parser.add_subparsers()

    parse_oneshot = parser.sub.add_parser('start', help='Play random episode.')
    parse_oneshot.set_defaults(which='start')
    parse_oneshot.add_argument('player', nargs='?', default='native')
    parse_oneshot.add_argument('playlist', nargs='?', default=None)


    parser.set_default_subparser('start')
    args = parser.parse_args(argv[1:])

    create_logger('random-episode', args.loglevel)

    if not getattr(args, 'which', None) or args.which == 'start':
        RandomEpisode(args.config, args.playlist, args.player).play_random()



from yaml import load

from random_episode.exceptions import ConfigException
from random_episode.players import exec_player

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper


class Config(dict):
    def __init__(self, file, **kwargs):
        super().__init__(**kwargs)
        self.file = file
        self.read()

    def read(self):
        self.update(load(open(self.file), Loader))
        self['playlists'] = self.get('playlists', {})
        self['players'] = self.get('players', {})
        if not 'chromecast' in self['players']:
            self['players']['chromecast'] = {'type': 'chromecast'}

    def get_playlist(self, playlist_name):
        if playlist_name is None:
            return
        if playlist_name not in self['playlists']:
            raise ValueError('{} is not a configured playlist.'.format(playlist_name))
        playlist = self['playlists'][playlist_name]
        if 'directories' not in playlist:
            raise ConfigException('You must specify a directories in the playlist {}'.format(playlist_name))
        return playlist

    def get_player(self, player_name):
        if player_name is None:
            return
        if player_name not in self['players']:
            raise ValueError('{} is not a configured player_name.'.format(player_name))
        return self['players'][player_name]

    def exec_player(self, path, player_name='native'):
        return exec_player(path, **self.get_player(player_name))



import os

from random_episode.config import Config
from random_episode.fs import get_random_episode


class RandomEpisode(object):
    def __init__(self, config_path, playlist_name=None, player_name='native'):
        self.config = Config(config_path)
        self.playlist = self.config.get_playlist(playlist_name)
        self.player_name = player_name

    def get_directories(self):
        if self.playlist is None:
            return [os.getcwd()]
        return self.playlist['directories']

    def get_random_episode(self):
        return get_random_episode(self.get_directories())

    def play_episode(self, episode):
        self.config.exec_player(episode.path, self.player_name)

    def play_random(self):
        episode = self.get_random_episode()
        self.play_episode(episode)



import random

from os3.fs.shortcuts import ls


def get_episodes(directories):
    all_episodes = []
    for directory in directories:
        all_episodes += list(ls(directory))
    return all_episodes


def get_random_episode(directories):
    return random.choice(get_episodes(directories))
